#!/usr/local/bin/perl
#***************************************************************************
#*          Copyright 2000 DBAmon Software, All Rights Reserved            *
#***************************************************************************
# --------------------------------------------------------------
# Changes:
#   BB 29Apr03 - Added largeTable code.
#   BB 08Apr05 - Added oddJob1
#   BB 05May05 - 2.00 - Added oddJob1
#   BB 03Jul06 - 2.01 - New logic to collect uptime and WWW display 
#   BB 21Mar08 - 2.02 - When ROOTPATH is null, logic was faulty 
# --------------------------------------------------------------
# Variables
# --------------------------------------------------------------
$version = "2.02"; 
$| = 1;

use Time::Local;
use File::Basename;

$P       = basename($0);
$progdir = dirname($0);
$tmpdir  = "$progdir/tmp";  
system("mkdir  $tmpdir 2>&1") if ! -e $tmpdir;
chdir("$progdir") || print "chdir to $progdir failed\n";
chop($pwd = `pwd`);

#---- START Temporary code to cleanup the old software/tmp files
system("rm /tmp/dbamon_* 2>/dev/null");
#---- END   Temporary code to cleanup the old software/tmp files

$tmpfl = "$tmpdir/dbamon_inf_tmp.txt";

# Method for sampling users: 1='onstat -u' 2='sysuserthreads'
$users_method = 2;

chop($thisos = `uname`);
if    ($thisos eq "HP-UX") { $bdfcmd = "bdf";    }
elsif ($thisos eq "SunOS") { $bdfcmd = "df -k"; }
elsif ($thisos eq "Linux") { $bdfcmd = "df -k"; }
else                       { $bdfcmd = "df -k";    }

chop($host = `hostname`);

$WWW = $ENV{"WWW"};

#---- Save Company name
chop($compwork = `nslookup $host | grep 'Name:' | awk '{ print \$2 }'`);
$fqhost = $compwork;

@words = split(/\./,$compwork);
$nexttolast = $#words - 1;
$company = $words[$nexttolast];

$verb = $ARGV[0];
$arg1 = $ARGV[1];
$arg2 = $ARGV[2];
$arg3 = $ARGV[3];
$arg4 = $ARGV[4];

$is = $ENV{"INFORMIXSERVER"};
$ifmxdir = $ENV{"INFORMIXDIR"};
#$informix_dbid = $ENV{"INFORMIX_DBID"};
$onconfig = $ENV{"ONCONFIG"};
$sqlhosts = $ENV{"INFORMIXSQLHOSTS"};

&msg("Starting - version=$version verb=$verb arg1=$arg1 arg2=$arg2 arg3=$arg3 arg4=$arg4");
&msg("  progdir=$progdir tmpdir=$tmpdir pwd=$pwd WWW=$WWW bdfcmd=$bdfcmd");
&msg("  company=$company fqhost=$fqhost env:");
system("env | grep INF");

#------------------------------------------------------------------------------
#
# Subroutines
#
#------------------------------------------------------------------------------

#------------------------------------------------------------------------------
sub msg
#------------------------------------------------------------------------------
{
  my $msgmsg = $_[0];
  chop($date = `date +%Y/%m/%d-%H:%M:%S`);
  print "$date ${P} | $msgmsg\n";
}

#------------------------------------------------------------------------------
sub error
#------------------------------------------------------------------------------
{
  $msg = $_[0];
  print "\n";
  print "$P | Fatal Error: $msg\n";
  print "$P | Possible SQL Diagnostics: \n";
  print $diagout;
  print "$P | Exiting\n";
  &msg("END_OF_PROBE");
  exit 1;
}

# ----------------------------------------------------------------------
# Count
# ----------------------------------------------------------------------
sub count
{
  if ($arg1 eq "Y")
  {
    $first = "
      select
	\"tables\", 
	count(*)
      from
	systabnames;";
  }

  $sql = "$first
      select
	\"databases\", count(*)
      from
	sysdatabases;

      select
	\"dblist\", name
      from
	sysdatabases;

      select
	\"dbspaces\", count(*)
      from
	sysdbspaces;

      select
	\"disks\", count(distinct fname) 
      from
	syschunks;

      select
	\"appldatabases\", count(*)
      from
	sysdatabases
      where 
	name not like \"sys%\";

      select 
	\"segshared\", 
	seg_osshmid 
      from 
	syssegments 
      where 
	seg_osshmid < 0;";

  #
  # Run dbaccess
  #

  system("echo '$sql' | dbaccess sysmaster");

  #
  # Run UX commands
  #

  print "HOSTNAME:       $host\n";

  system("onstat -V");
  system("uname -a");

  if ($thisos eq "SunOS") 
  { 
    chop($cpu = `uname -a | head -1 | awk '{ print \$7 }'`);
    print "CPUTYPE: $cpu \n";
    chop($osver = `uname -a | head -1 | awk '{ print \$3 }'`);
    print "OS_VERSION: $osver \n";
  }
  if ($thisos eq "HP-UX") 
  { 
    system("$progdir/dbamon_cpuinfo"); 
    system("echo 'MODE' `getconf SC_KERNEL_BITS 2>&1`");
    system("model");
  }

  # Measure how full INFORMIXDIR FS is
  $bdfflag = "OK";
  chop($bdf = `$bdfcmd $ifmxdir | tail -1`);
  ##print "bdf=$bdf ifmxdir=$ifmxdir \n";
  $_ = $bdf;
  @words  = split;
  if ($#words == 5 && index($bdf,"%") > -1)
  {
    $bdfsize = int($words[1]/1000);
    $bdfused = int($words[2]/1000);
    $bdfaval = int($words[3]/1000);
    chop($bdfpct  = $words[4]);
    $bdfname = $words[5];
  }
  elsif ($#words == 4 && index($bdf,"%") > -1)
  {
    $bdfsize = int($words[0]/1000);
    $bdfused = int($words[1]/1000);
    $bdfaval = int($words[2]/1000);
    chop($bdfpct  = $words[3]);
    $bdfname = $words[4];
  }
  else
  {
    $bdfflag = "E";
  }
  $bdfflag = "E" if $bdfsize eq "";
  $bdfflag = "C" if $bdfpct >= 99;
  print "BDFIFMXDIR: $bdfsize $bdfused $bdfaval $bdfpct $bdfname $bdfflag \n";
}

# ----------------------------------------------------------------------
# DBS
# ----------------------------------------------------------------------
sub dbs
{
  # See if we are running version 5
  if ( -e "${ifmxdir}/bin/tbinit")
  {
    $ver5 = 1;
    print "$P | This is Informix version 5\n";
  }

  if ($ver5)
  {
    system("$progdir/dbamon_spaces");
    return;
  }
  else
  {
    $stmt = "
    unload to '$tmpfl' 
    select
      a.name,
      a.nchunks chks,
      sum(b.chksize) size,
      sum(b.nfree) free
    from
      sysdbspaces a,
      syschunks b
    where
      a.dbsnum = b.dbsnum
    group by
      a.dbsnum,
      a.name,
      a.nchunks
    order by 1";
    ##print "stmt=$stmt \n";
    print "$P | Running dbaccess to query dbspace data \n";
    $diagout = `echo '$stmt' | dbaccess -e sysmaster 2>&1`;
    open(IN, "$tmpfl") || &error("Cannot open input $tmpfl");
    while(<IN>)
    {
      ##print;
      $diagout .= $_;

      @words = split(/\|/,$_);

      $dbsname = $words[0];
      $chunks  = $words[1];

      $size    = int($words[2]*2048/1000000);
      $free    = int($words[3]*2048/1000000);

      $used    = int($size - $free);
      $pct     = int(($used / $size) * 100);
      printf("THIS: %-12s %3d %5d %5d %5d %3d\n",$dbsname,$chunks,$size,$free,$used,$pct);
    }
    close(IN);
  }

  print "DIAGOUT: $diagout \n" if $dbsname eq "";

  system("onstat -c | egrep '^CLEANERS|^LRUS|^SHMVIRTSIZE|^WSTATS|^LTAPEDEV|^NUMAIOVPS'");
  system("onstat -g iov | grep 'kio'");

  if ($thisos eq "HP-UX") { $chatrcmd = "chatr";    }
  if ($thisos eq "SunOS") { $chatrcmd = "/usr/bin/dump -Lv"; }
  else                    { $chatrcmd = "chatr";    }
  system("$chatrcmd \$INFORMIXDIR/bin/oninit | grep -i shmem_magic");

  $aiotop = `onstat -g iov | grep aio | head -1 | awk '{ print \$5 }'`;
  $aiobot = `onstat -g iov | grep aio | tail -1 | awk '{ print \$5 }'`;
  $aionum = `onstat -g iov | grep aio | wc -l'`;
  chop $aiotop;
  chop $aiobot;
  chop $aionum;

  if ($aiotop > 0)
  {
    $aiorat = int(($aiobot*100)/$aiotop);
  }
  else
  {
    $aiorat = -1;
  }

  print "AIORAT: $aiorat Top: $aiotop Bot: $aiobot Num: $aionum\n";

  #
  # If an SAP Informix instance, return SAP version
  #

  $ifmxdirl = length($ifmxdir);
  $ifmxdirs = substr($ifmxdir,0,10);

  ##print "$ifmxdir $ifmxdirl $ifmxdirs\n";
  if (($ifmxdirl == 13 && $ifmxdirs eq "/informix/"))
  {
    #
    # Get SAP kernel ver
    #

    $sapinst = substr($ifmxdir,10,3);
    ##print "$sapinst\n";

    $sapkern = `/usr/sap/$sapinst/SYS/exe/run/R3check -? | grep version | head -1 | awk '{ print \$7 }'`;
    chop $sapkern;

    $sapkernmode = `/sapmnt/$sapinst/exe/disp+work 2>/dev/null | grep 'compiled for' | awk '{ print \$3 }'`;
    chop $sapkernmode;
    if ($sapkernmode eq "")
    {
      $sapkernmode = "32";
    }
    $sapkern = "$sapkern($sapkernmode)";

    $saprlse = `echo "select * from svers" | dbaccess $sapinst 2>/dev/null | tail -3 | head -1`;
    chop $saprlse;

    print "SAP: $sapkern $saprlse\n";
  }
}

#------------------------------------------------------------------------------
sub timeToText
#------------------------------------------------------------------------------
{
  $zz_ts = $_[0];
  ($sec,$min,$hour,$mday,$mon,$year) = localtime($zz_ts);
  $year += 1900;
  $mon += 1;

  # Pad with 0's
  $yy = sprintf("%04d",$year);
  $mm = sprintf("%02d",$mon);
  $dd = sprintf("%02d",$mday);
  $hh = sprintf("%02d",$hour);
  $mi = sprintf("%02d",$min);
  $ss = sprintf("%02d",$sec);

  $date = "$yy/$mm/$dd-$hh:$mi:$ss";
  return $date;
}

# ----------------------------------------------------------------------
sub wwwDisplay
# ----------------------------------------------------------------------
{
  if ($ifmx_status eq "On-Line")
  {
    $www_bgcolor = "lime";
  } 
  else
  {
    $www_bgcolor = "pink";
  }

  &msg("WWW Status Summary:");
  print "<table border=0 \n";
  print "<tr bgcolor=slate><td><font size=2>Item<td><font size=2>Status\n";
  print "<tr bgcolor=$www_bgcolor><td><font size=2>Informix Status<td><h5>$ifmx_status\n";
  print "<tr bgcolor=$www_bgcolor><td colspan=2>\n";
  print "<tr bgcolor=$www_bgcolor><td><font size=2>Hostname<td><h5>$host\n";
  print "<tr bgcolor=$www_bgcolor><td><font size=2>Instance Started:<td><h5>$stime_text\n";
  print "<tr bgcolor=$www_bgcolor><td><font size=2>Instance Hours Up:<td><h5>$hours_up\n";
  print "<tr bgcolor=$www_bgcolor><td><font size=2>Instance Days Up:<td><h5>$days_up\n";
  print "<tr bgcolor=$www_bgcolor><td><font size=2>Master 'oninit' Pid:<td><h5>$master_init_pid\n";
  print "</table><br>\n";

  &msg("Informix Message Log:"); 
 
  print "<table bgcolor=$www_bgcolor><tr><td><pre>\n"; 

  foreach (`onstat -m`) 
  {
    print;
  }

  print "</pre></table>\n"; 
}

# ----------------------------------------------------------------------
sub calcUptime
# ----------------------------------------------------------------------
{
  $ENV{"UNIX95"} = 1;

  chop($master_init_pid = `onstat -g sch | grep cpu | head -1 | awk '{ print \$2 }'`); 
  &msg("calcUptime - Calculating instance uptime - master_init_pid=$master_init_pid"); 
  system("ps -ef -oetime,pid,tty,comm,ruser | grep '$master_init_pid' | egrep -v 'sh|grep'");

  open(PS, "ps -ef -oetime,pid,tty,comm,ruser | grep ' oninit' | egrep -v 'sh|grep' |") || die "can't run ps - $!";

  while(<PS>)
  {
    ##print;
    @words = split;
    $etime = $words[0];
    $pid   = $words[1];

    next unless $pid == $master_init_pid;

    ##&msg("found pid=$pid etime=$etime"); 

    ## @t = split(":-",$etime);
    @t = split(/[-:]/,$etime);

    $secs = -1;
    if (length($etime) == 5)  { $secs = ($t[0] * 60) + $t[1]; }
    if (length($etime) == 8)  { $secs = ($t[0] * 3600) + ($t[1] * 60) + $t[2]; }
    if (length($etime) >= 10) { $secs = ($t[0] * 86400) + ($t[1] * 3600) + ($t[2] * 60) + $t[3]; }
    #if (length($etime) == 11) { $secs = ($t[0] * 86400) + ($t[1] * 3600) + ($t[2] * 60) + $t[3]; }
    #if (length($etime) == 12) { $secs = ($t[0] * 86400) + ($t[1] * 3600) + ($t[2] * 60) + $t[3]; }

    $days_up  = sprintf("%.2f", $secs / (24*60*60));
    $hours_up = sprintf("%.2f", $secs / (60*60));

    $stime_num = time - $secs;
    $stime_text = &timeToText($stime_num);

    ##print "pid=$pid stime_num=$stime_num stime_text=$stime_text secs=$secs days=$days\n";

    last;
  }
  close(PS);

  #---- Gather instance status
  @onstat_fields = split(/ /,`onstat - | grep -v '^\$'`);

  #foreach (@onstat_fields)
  #{
  #  print "($_)";
  #}

  if ($onstat_fields[4] =~ /^7/)
  {
    &msg("Informix V7 family"); 
    $ifmx_status = $onstat_fields[10]; 
  }
  elsif ($onstat_fields[5] =~ /^7/)
  {
    &msg("Informix V7 family"); 
    $ifmx_status = $onstat_fields[11]; 
  }
  elsif ($onstat_fields[4] =~ /^7.31/)
  {
    &msg("Informix V7 family"); 
    $ifmx_status = $onstat_fields[7]; 
  }
  elsif ($onstat_fields[2] =~ /^7.24/)
  {
    &msg("Informix V7.24 family"); 
    $ifmx_status = $onstat_fields[7]; 
  }
  elsif ($onstat_fields[2] =~ /^5/)
  {
    &msg("Informix V5 family"); 
    $ifmx_status = $onstat_fields[6]; 
  }
  elsif ($onstat_fields[5] =~ /^9.21/)
  {
    &msg("Informix V9.21 family"); 
    $ifmx_status = $onstat_fields[11]; 
  }
  elsif ($onstat_fields[5] =~ /^9/)
  {
    &msg("Informix V9 family"); 
    $ifmx_status = $onstat_fields[9]; 
  }
  else 
  {
    &msg("Unknown Informix family"); 
  }

  chop($calc_date = `date +%Y/%m/%d-%H:%M:%S`);
  $calc_time = time;

  if ($WWW ne "zY")
  {
    if ($master_init_pid ne "")
    {
      print "$P MASTER_INIT_PID: $master_init_pid\n"; 
      print "$P STIME_NUM:       $stime_num\n"; 
      print "$P STIME_TEXT:      $stime_text\n"; 
      print "$P HOURS_UP:        $hours_up\n"; 
      print "$P DAYS_UP:         $days_up\n"; 
      print "$P UPTIME:          $stime_text $stime_num $calc_date $calc_time\n" if $stime_text; 
    }
  }
}

# ----------------------------------------------------------------------
# Status
# ----------------------------------------------------------------------
sub status
{
  #
  # Run onstat - to get status
  #
  $rc = system("onstat -"); 

  # See if we are running version 5
  if ( -e "${ifmxdir}/bin/tbinit")
  {
    print "$P | This is Informix version 5 - skipping SQL check\n";
    return;
  }

  # Try to run sql
  print "$P | Running dbaccess to ensure that SQL will run\n";
  open(IN, "echo '\
    set isolation to dirty read;
    select \"FLAG:\", count(*) from sysdatabases;
  ' | dbaccess sysmaster 2>&1 |") || &error("Error running dbaccess");
  $flag = 0;
  while(<IN>)
  {
    $diagout .= $_;
    @words = split;
    $verb    = $words[0];
    if ($verb eq "FLAG:") { $flag = 1; }
  }

  if ($flag == 0)
  {
    print "SQLWONTRUN: $diagout \n";
  }

  # Collect sessions HWM data
  if ($users_method == 1)
  {
    chop($hwm = `onstat -u | grep 'maximum concurrent' | awk '{ print \$5 }'`);
  }
  elsif ($users_method == 2)
  {
    print "$P | Running dbaccess to count the current number of non-informix threads\n";
    open(IN, "echo '\
      set isolation to dirty read;
      select \"FLAG:\", count(us_name) from sysuserthreads where us_name not matches \"informix\";
    ' | dbaccess sysmaster 2>&1 |") || &error("Error running dbaccess - sysuserthreads");
    $flag = 0;
    while(<IN>)
    {
      print;
      $diagout .= $_;
      @words = split;
      $verb    = $words[0];
      if ($verb eq "FLAG:") 
      { 
        $hwm = $words[1]; 
      }
    }
    close(IN);
  }
  print "SESSIONSHWM: $hwm\n";

  # Look for critical files
  if (-r "${ifmxdir}/etc/$onconfig" && -s "${ifmxdir}/etc/$onconfig")
  {
    print "DBFILECHECKOK: ONCONFIG: ${ifmxdir}/etc/$onconfig \n";
  }
  else
  {
    print "DBFILECHECKNG: ONCONFIG: ${ifmxdir}/etc/$onconfig \n";
  }

  if (-r "${ifmxdir}" && -d "${ifmxdir}")
  {
    print "DBFILECHECKOK: INFORMIXDIR: ${ifmxdir} \n";
  }
  else
  {
    print "DBFILECHECKNG: INFORMIXDIR: ${ifmxdir} \n";
  }
  if ($sqlhosts ne "")
  {
    if (-r "${sqlhosts}" && -s "${sqlhosts}")
    {
      print "DBFILECHECKOK: INFORMIXSQLHOSTS: ${sqlhosts} \n";
    }
    else
    {
      print "DBFILECHECKNG: INFORMIXSQLHOSTS: ${sqlhosts} \n";
    }
  }

  # See if the root chunk is readable
  &msg("Determining ROOTPATH");
  chop($rootpath = `onstat -c | grep '^ROOTPATH' | awk '{ print \$2 }'`);
  &msg("Derived rootpath=$rootpath");

  if ($rootpath eq "")
  {
    &msg("ROOTPATH is NULL - something is wrong (onstat -c did not return the ROOTPATH value)");
  }
  elsif (! -r "${rootpath}")
  {
    &msg("ERROR: ROOTPATH file=$rootpath Is not readable by this UX userid");
    system("ll $rootpath 2>&1");
    system("id");
    print "ROOTPATHCHECKNG: $rootpath\n";
  }
  else
  {
    &msg("ROOTPATH file=$rootpath Is readable");
  }
}

# ----------------------------------------------------------------------
# HDR
# ----------------------------------------------------------------------
sub hdr
{
  #
  # Run onstat - to get status
  #
  open(IN, "echo '\
  set isolation to dirty read;
  select
    *
  from
    sysdri
  ' | dbaccess sysmaster 2>/dev/null |") || &error("Error running dbaccess");

  while(<IN>)
  {
    @words = split;
    $verb    = $words[0];
    print;
  }
}

# ----------------------------------------------------------------------
# Locks
# ----------------------------------------------------------------------
sub locks
{
  #
  # find number of locks currently held
  #

  open(IN, "echo '\
  set isolation to dirty read;

  select
    \"LOCKSHELD\",
    sum(nlocks)
  from
    systxptab
  ;

  --
  -- A bug in 7.30.UC10* makes bad values in lockheld of syssesprof
  --

  select
    cf_name,
    cf_effective
  from
    sysconfig
  where
    cf_name = \"LOCKS\";

  select
    \"ARCTYPE\",
    count(*)
  from
    arc_request;

  select
    \"BARARC\",
    count(*)
  from
    sysutils:bar_action;

  select
    \"SESSION\",
    sid,
    locksheld
  from
    syssesprof
  where
    locksheld > 2
    and sid not in (select sid from syssessions where pid < 0)
    and locksheld <= (select cf_effective from sysconfig where cf_name = \"LOCKS\")
    and locksheld != 65536
    and locksheld != 131072
  order by
    3 ;
  select 
    \"SHMVSEGCOUNT:\", 
    count(*),
    sum(seg_size)
  from 
    syssegments 
  where 
    seg_class = 2
  ' | dbaccess sysmaster 2>&1 |") || &error("Error running dbaccess");

  while(<IN>)
  {
    ##print;
    $diagout .= $_;

    @words = split;
    $verb    = $words[0];
    if ($verb eq "SHMVSEGCOUNT:")
    {
      print;
    }
    if ($verb eq "LOCKSHELD")
    {
      $locksheld = $words[1];
    }
    if ($verb eq "cf_effective")
    {
      $locksmax  = $words[1];
      $lockspct = int($locksheld / $locksmax * 100);
      print "LOCKSPCT: $lockspct $locksheld $locksmax\n";
    }
    if ($verb eq "SESSION")
    {
      print;
    }
    if ($verb eq "ARCTYPE")
    {
      print;
    }
    if ($verb eq "BARARC")
    {
      print;
    }
  }

  if ($locksheld eq "")
  {
    &error("dbaccess Error: LOCKSHELD is null");
  }

  #
  # Sample ckpt time
  #

  open(IN, "onstat -m | grep 'Checkpoint Completed' |") || &error("Error running onstat -m");

  while(<IN>)
  {
    ##print;
    @words = split;
    $dur  = $words[5];
    $sum += $dur;
    $cnt += 1;
    ##print "dur=$dur\n";
  }

  ##print "sum=$sum\n";
  ##print "cnt=$cnt\n";
  if ($cnt eq "")
  {
    $avg = -1;
  }
  else
  {
    $avg = $sum/$cnt;
  }
  printf("CKPTAVG: %4.1f\n",$avg);

  #
  # Sample maxusers
  #

  system("onstat -u | grep 'maximum concurrent' | awk '{ print \"MAXUSERS: \" \$5 }'");
  system("onstat -u | egrep 'F |D |B |M' | wc -l | awk '{ print \"SYSUSERS: \" \$1 }'");

  #
  # Sample NFILE usage
  #

  open(IN, "sar -v 1 | tail -1 |") || &error("Error running sar -v");

  while(<IN>)
  {
    ##print;
    @words     = split;
    $nfileraw  = $words[7];
    ##print "nfileraw=$nfileraw\n";
    @words2    = split(/\//,$nfileraw);
    $current   = $words2[0];
    $max       = $words2[1];
    $pct       = ($current/$max*100);
  }
  printf("NFILE: %4d %4d %4d\n",$current,$max,$pct);
}

# ----------------------------------------------------------------------
sub onarc_report
# ----------------------------------------------------------------------
{
  open(INDB, "echo '\
  set isolation to dirty read;
  select
    \"FLAG\" z,
    b.req_execution_date,
    a.ae_level,
    b.req_rid,
    c.ds_name, 
    current year to day - b.req_execution_date daysago 
  from
    arc_archive_event a,
    arc_request b,
    arc_dbspace_set c
  where
    a.ae_rid = b.req_rid
    and b.req_status = \"SU\" 
    and b.req_type = \"AR\"
    and a.ae_dsid = c.ds_dsid
    and b.req_execution_date >= current - 45 units day
  order by
    2 desc into temp tonarc;
  select * from tonarc ;
  ' | dbaccess -e sysmaster 2>&1 |") || &error("Error running dbaccess");

  printf "\n\nRPT: Report of all ONArchive Completed Archive Requests - Last 45 Days\n\n";
  print "RPT: Backup Date              Level   Request ID   DBSpace Set  Days Old\n";
  print "RPT: ----------------------   -----   ----------   -----------  --------\n";

  while(<INDB>)
  {
    ##print;
    @words = split;
    $verb    = $words[0];
    if ($verb eq "FLAG")
    {
      printf "RPT: %-12s %-5s %8s %12s   %-12s     %4d\n",$words[1],$words[2],$words[3],$words[4],$words[5],$words[6];
      $daysago = $words[6];
      $level = $words[3];
      ##print "mfl0daysago=$mrl0daysago level=$level \n";
      if ($mrl0daysago eq "" && $level eq "0")
      {
        $mrl0daysago = $daysago;
      }
    }

  }
  close(INDB);
  print "\nMRL0DAYSAGO: $mrl0daysago\n";
  printf "\nNote: dbspaceset=* backups will not appear on this report\n\n";
}

# ----------------------------------------------------------------------
sub onarc_listlast
# ----------------------------------------------------------------------
{
  open(INDB, "echo '\
  select
    *
  from
    arc_request a,
    arc_archive_event b
  where
    a.req_rid = b.ae_rid and
    b.ae_level = 0 and
    req_type = \"AR\"
  into temp t1
  ;
  select
    \"MRL0DSID\",
    ae_dsid
  from
    t1
  where
  req_execution_date =
    (select max(req_execution_date) from t1)
    ;
  select 
    *,
    current - req_execution_date DIFF
    from 
    t1;
  ' | dbaccess -e sysmaster 2>&1 |") || &error("Error running dbaccess");

  while(<INDB>)
  {
    ##print;
    @words = split;
    $verb    = $words[0];

    if ($verb eq "MRL0DSID")
    {
      #$mrl0dsid = $words[1];
    }
    if ($verb eq "diff")
    {
      $mrl0daysago = $words[1];
      #last;  # Mistake???
    }
  }
  close(INDB);
}


# ----------------------------------------------------------------------
sub onarc_listarc
# ----------------------------------------------------------------------
{
  open(INDB, "echo '\
  set isolation to dirty read;
  select
    a.ae_level,
    a.ae_dsid,
    a.ae_timestamp,
    b.req_execution_date,
    current now,
    current - b.req_execution_date DIFF
  from
    arc_archive_event a,
    arc_request b
  where
    a.ae_rid = b.req_rid
    and b.req_execution_date =
    (select max(req_execution_date) from arc_request
     where req_status = \"SU\" and req_type = \"AR\")
    ;
  ' | dbaccess -e sysmaster 2>&1 |") || &error("Error running dbaccess");

  while(<INDB>)
  {
    ##print;
    @words = split;
    $verb    = $words[0];

    if ($verb eq "diff")
    {
      $ts      = $words[2];
      @words2  = split(/:/,$ts);
      $in_days = int($words[1]);
      $in_hrs  = int($words2[0]);

      $hour    = $in_hrs + ($in_days * 24);

      ##print "ts=$ts hour=$hour\n";
    }

  }
  close(INDB);
}


# ----------------------------------------------------------------------
sub onarc_running
# ----------------------------------------------------------------------
{
  open(INDB, "echo '\
  set isolation to dirty read;
  select
    \"THIS\",
    b.req_execution_date
  from
    arc_request b
  where
    req_status = \"EX\" and req_type = \"AR\"
    ;
  ' | dbaccess -e sysmaster 2>&1 |") || &error("Error running dbaccess");

  $running = "N";

  while(<INDB>)
  {
    ##print;
    @words = split;
    $verb    = $words[0];
    if ($verb eq "THIS")
    {
      $running = "Y";
    }
  }
  close(INDB);
}


# ----------------------------------------------------------------------
sub onarc
# ----------------------------------------------------------------------
{
  #
  # Find DSID of most recent backup
  #
  #&onarc_listlast();
  #print "MRL0DSID: $mrl0dsid\n";
  #print "MRL0DAYSAGO: $mrl0daysago\n";

  #
  # Find number of hours since any backup
  #
  &onarc_listarc();
  print "HOURS: $hour\n";

  &onarc_running();
  print "RUNNING: $running\n";

  &onarc_report();
}


# ----------------------------------------------------------------------
sub dup
# ----------------------------------------------------------------------
{
  open(IN, "onstat -c | grep -v '^\$' | awk '{ print \$1 }' | grep -v '^#' | sort |") || &error("Error running onstat");

  $lastverb = "crudlaksjlaskdjalksjd";

  while(<IN>)
  {
    ##print;
    @words = split;
    next if $#words == -1;
    if ($words[0] eq $lastverb && $words[0] ne "NETTYPE")
    {
      print "Dup: $words[0]\n";
    }
    $lastverb = $words[0];
  }
}
  

# ----------------------------------------------------------------------
sub gettime
# ----------------------------------------------------------------------
{
  print "SystemTime: " . time() . "\n";
}


# ----------------------------------------------------------------------
sub largeTables
# ----------------------------------------------------------------------
{
  $large_threshold_gb = 25;
  $gb_1 = 1073741824;

  print "$P | largeTables - Looking for objects > $large_threshold_gb gB in size\n";

  # Non-fragmented tables
  open(IN,"echo '\
  set isolation to dirty read;

  select 
    \"X\",
    tabname, 
    partnum, 
    sum(te_size)*2048
  from 
    systabnames, 
    systabextents
  where 
    partnum = te_partnum
  group by 
    2,3
  having
    sum(te_size)*2048 > 25000000000
  order by 
    4 desc;
  ' | dbaccess sysmaster 2>&1 |") || &error("Error opening cmdout");

  while(<IN>)
  {
    ##print;
    @words = split;

    $verb    = $words[0];
 
    if ($verb eq "X")
    {
      $obj     = $words[1];
      $partnum = $words[2];
      $size    = $words[3];

      $size = sprintf("%.2f",$size / $gb_1);
      ##print "verb=$verb obj=$obj partnum=$partnum size=$size\n";

      next if $size < $large_threshold_gb;

      print "$P LARGEOBJECT: $obj $partnum $size\n";
    }
  }

}

# ----------------------------------------------------------------------
sub longtx 
# ----------------------------------------------------------------------
{
  $debug = 0;

  #
  # Find ltxhwm
  #

  $ltxhwm = `onstat -c | grep ^LTXHWM | awk '{ print \$2 }'`;
  chop $ltxhwm;
  if ($debug) { print "ltxhwm=$ltxhwm\n"; }

  #
  # Find current log #
  #

  $log_cur = `onstat -l | grep 'C' | grep -vi INFORMIX | awk '{ print \$4 }'`;
  chop $log_cur;
  if ($debug) { print "log_cur=$log_cur\n"; }

  #
  # Find number of logs
  #

  $log_cnt = `echo "select 'THIS', count(*) from syslogs" | dbaccess sysmaster 2>&1 | grep THIS | awk '{ print \$2 }'`;
  chop $log_cnt;
  if ($debug) { print "log_cnt=$log_cnt\n"; }

  #
  # Find tx's with non-zero logbegin  
  #

  open(IN, "echo '\
  set isolation to dirty read;
  select 
    tx_id, 
    us_name, 
    us_sid, 
    username,
    pid,
    hostname,
    connected,
    tx_logbeg 
  from 
    systrans,
    sysuserthreads,
    syssessions
  where 
    tx_logbeg > 0
    and us_address = tx_owner
    and us_sid = sid
  ;' | dbaccess sysmaster 2>&1 |") || &error("Error running dbaccess");

  print "Possible Long Transactions\n";
  print "Current Log: $log_cur  LTXHWM: $ltxhwm Log Count: $log_cnt\n\n";
  print "   TX-Id User Name SID     Hostname           PID   Connected          LogBeg LogDlta  LogPct  HWMPct \n";

  while(<IN>)
  {
    if ($debug) { print };
    @words = split;
    $verb    = $words[0];

    if ($verb eq "tx_id")     { $tx_id     = $words[1]; }
    if ($verb eq "us_name")   { $us_name   = $words[1]; }
    if ($verb eq "us_sid")    { $us_sid    = $words[1]; }
    if ($verb eq "hostname")  { $hostname  = $words[1]; }
    if ($verb eq "pid")       { $pid       = $words[1]; }
    if ($verb eq "connected") 
    { 
      $connected = $words[1]; 
      ($sec,$min,$hour,$mday,$mon,$year)=localtime($connected);
      ++$mon;
      $cts = "$year/$mon/$mday-$hour:$min:$sec";
    }
    

    if ($verb eq "tx_logbeg") 
    { 
      $tx_logbeg = $words[1]; 

      $log_delta = $log_cur - $tx_logbeg;
      $log_pct   = int($log_delta / $log_cnt * 100);
      $log_hwm   = int($log_pct / $ltxhwm * 100);
      if ($debug) { print "log_delta=$log_delta log_pct=$log_pct\n"; }

      printf("-- %-4s  %-8s  %-6s  %12s  %6s %-17s  %6s  %6s  %6s  %6s\n",$tx_id, $us_name, $us_sid, $hostname, $pid, $cts, $tx_logbeg, $log_delta, $log_pct, $log_hwm);

      #
      # If the option was specified, get the SQL too
      #
      if ($arg1 eq "-sql")
      {

	open(INSQL, "echo '\
	set isolation to dirty read;
	select
	  sqs_dbname,
	  sqs_statement
	from
	  syssqlstat
	where
	  sqs_sessionid + 0 = \"$us_sid\"
	;' | dbaccess sysmaster 2>&1 |") || &error("Error running dbaccess #2");
	while (<INSQL>)
	{
	  if ($debug) { print };
	  @words = split;
	  $verb    = $words[0];

	  if ($verb eq "")
	  {
	    $flag = 0;
	  }
	  if ($flag) 
	  {
	    print;
	  }
	  if ($verb eq "sqs_dbname")
	  {
	    print "    DBName: $words[1]\n";
	  }
	  if ($verb eq "sqs_statement")
	  {
	    $s = substr($_,15);
	    print "    Statement: $s";
	    $flag = 1;
	  }
	} # End of while
      print "\n";
      } # End of if sql
    }
  }
}

# ----------------------------------------------------------------------
sub bkrun
# ----------------------------------------------------------------------
{
  $cmd = $arg1;
  print "$P bkrun: Issuing at now command to run backup:\n";
  print "$P bkrun: Command before {SID} substitution: $cmd\n";
  $cmd =~ s/\{SID\}/$is/g;
  $cmd = "echo 'time $cmd' | at now";
  print "$P bkrun: About to issue: $cmd\n";
  $rc = system($cmd);
  print "$P bkrun: Back from issuing at now - rc=$rc\n";
}

# ----------------------------------------------------------------------
sub filecheck
# ----------------------------------------------------------------------
{
  $cfg = "$progdir/dbamon_inf.cfg";
  chop($fc_filename = `grep '^FILECHECK' $cfg | awk '{ print \$2 }'`);
  if ($fc_filename eq "")
  {
    print "$P FILECHECK: Parm not found in $cfg\n";
    return;
  }
  print "$P FILECHECK: fc_filename=$fc_filename\n";
  if (! -e $fc_filename)
  { 
    print "$P FILECHECK: NotFound fc_age_hours: . $fc_filename\n";
    return;
  }
  $fc_mtime = (stat "$fc_filename") [9];
  $fc_age = time - $fc_mtime;
  $fc_age_hours = $fc_age / 3600;
  $fc_age_hours = sprintf("%.2f", $fc_age_hours);
  print "$P FILECHECK: Found fc_age_hours: $fc_age_hours $fc_filename\n";
}

# ----------------------------------------------------------------------
sub aliasInfo
# ----------------------------------------------------------------------
{
  &msg("aliasInfo() - Starting");
  # Send host/alias info
  print "$P HOSTNAME:       $host\n";

  @alias_list = `netstat -i | grep ':' | grep 'com \$' | awk '{ print \$NF }'`;
  foreach (@alias_list)
  {
    chop;
    print "$P NETALIAS:       $_\n";
  }
  &msg("aliasInfo() - Done");
}

# ----------------------------------------------------------------------
sub oddJob1 
# ----------------------------------------------------------------------
{
  $thiskey = $_[0];
  $sshdir  = "$ENV{'HOME'}/.ssh";
  $keyfile = "$sshdir/authorized_keys";
  &msg("oddJob1() Starting - thiskey=$thiskey sshdir=$sshdir keyfile=$keyfile");

  #---- Set correct permissions
  system("chmod 755 $sshdir 2>&1"); 
  system("chmod 644 $keyfile 2>&1"); 
 
  #---- See if sshdir exists - if not, create it 
  &msg("  Seeing if $sshdir exists");
  if (! -e "$sshdir")
  {
    &msg("  $sshdir Does not exist - Creating");
    $rc = system("mkdir $sshdir 2>&1"); 
    if ($rc != 0)
    {
      system("echo ' ' | mailx -s 'DBAmon | $host/$os - mkdir $sshdir no workie - $!' dbaadmin\@dba.it.agilent.com");
    }
    system("echo ' ' | mailx -s 'DBAmon | $host/$os - Created $sshdir' dbaadmin\@dba.it.agilent.com");
  }

  #---- See if authkeys file contains this entry - if not, add it 
  &msg("  Looking in $keyfile for $thiskey DSA entry");
  if (system("grep 'ssh-dss' $keyfile | grep '$thiskey' > /dev/null") != 0)
  {
    &msg("  $keyfile Does not contain a DSA entry for $thiskey - Adding entry");
    $rc = system("grep '$thiskey' $progdir/dbamon_ora.dsa1 >> $keyfile 2>&1"); 
    if ($rc != 0)
    {
      system("echo ' ' | mailx -s 'DBAmon | $host/$os - Add $thiskey no workie - $!' dbaadmin\@dba.it.agilent.com");
    }
    system("echo ' ' | mailx -s 'DBAmon | $host/$os - Added DSA $thiskey to $keyfile' dbaadmin\@dba.it.agilent.com");
  }
  
  &msg("oddJob1() Done");
}

# ----------------------------------------------------------------------
#
# Main
#
# ----------------------------------------------------------------------

$stime = time;

#
# Check to see if locks are being exceeded
#
if ($verb eq "locks") 
{
  $subrc = &locks();
}

#
# Check Informix status
#
if ($verb eq "status") 
{
  $subrc  = &status();
  $subrc += &calcUptime();
  $subrc += &filecheck();
  #$subrc+= &largeTables();
  $subrc += &aliasInfo();
  if ($company eq "agilent")
  {
    &oddJob1("dbamon\@bigdog");
    &oddJob1("dbamon\@lvmon01");
  }
}

if ($verb eq "www_status") 
{
  $subrc  = &calcUptime();
  $subrc  = &wwwDisplay();
}

#
# Count tables
#
if ($verb eq "count") 
{
  $subrc = &count();
}

#
# Check DBSpaces
#
if ($verb eq "dbs") 
{
  $subrc = &dbs();
}

#
# Check HDR
#
if ($verb eq "hdr") 
{
  $subrc = &hdr();
}

#
# Check ONarchive
#
if ($verb eq "onarc") 
{
  $subrc = &onarc();
}

#
# Check for dup onconfig parms
#
if ($verb eq "dup") 
{
  $subrc = &dup();
}

#
# Get the current time
#
if ($verb eq "gettime") 
{
  $subrc = &gettime();
}

#
# Check for longtx's
#
if ($verb eq "longtx") 
{
  $subrc = &longtx();
}

#
# Run a backup
#
if ($verb eq "bkrun")
{
  $subrc = &bkrun();
}

#---- Testing
if ($verb eq "odd1test")  { $subrc = &oddJob1("dbamon\@bigdog"); }

#
# Terminate
#
$etime = time;
$elapsed = $etime - $stime;
print "$P | Done - elapsed: $elapsed - subrc: $subrc\n";

&msg("END_OF_PROBE");
exit 0;
